import epics
import time
import json
import os
import signal
import matplotlib.pyplot as plt
from git import Repo
from shutil import rmtree
from glob import glob
from numpy import linspace, array, zeros, flip, rot90, tile
from http.server import BaseHTTPRequestHandler,HTTPServer
from epicsarchiver import ArchiverAppliance
from channelfinder import ChannelFinderClient
from threading import Thread, Event

from screens.pos import posScreen
from screens.interlocks import interlocksScreen
from screens.instruments import instrumentsScreen
from screens.ts2_cav import ts2_cavScreen
from screens.ts2_screens import ts2_screensScreen
from screens.ts2_tpcircuits import ts2_tpcircuitsScreen

PORT_NUMBER = 8080
ARCHIVER_SERVER = os.environ.get('ARCHIVER_SERVER')
archiver = ArchiverAppliance(ARCHIVER_SERVER)
CHANNEL_FINDER = os.environ.get('CHANNEL_FINDER')
cf = ChannelFinderClient(BaseURL=CHANNEL_FINDER)
stop_signal = Event()
stop_signal.clear()

def save_json(json_data, outfile):
    tmp_json = json.dumps(json_data)
    tmp_json = tmp_json.replace('NaN','0')
    tmp_json = tmp_json.replace('Infinity','0')
    with open(outfile,'w') as datafile:
        datafile.write(tmp_json)
    return

all_casings=lambda s:s and {r[0]+t for r in{s,s.swapcase()} for t in all_casings(s[1:])} or{s}

class myHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        self.send_response(200)
        path = list(filter(None,self.path.split('/')))
        if len(path) == 2:
            if path[0] == 'search':
                returnlist=''
                self.send_header('Content-type','application/json')
                self.send_header('Access-Control-Allow-Origin', '*')
                self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
                self.end_headers()
                wordlist=path[1].split("%20")
                if len(wordlist)<=1 and len(wordlist[0])<2:
                    returnlist=''
                    return
                search_string=''
                if sum(len(i) for i in wordlist) <= 10:
                    for word in wordlist:
                        for case in all_casings(word):
                            search_string+='*'+case+'*|'
                        search_string=search_string[:-1]
                        search_string+=','
                    search_string=search_string[:-1]
                else:
                    for word in wordlist:
                        search_string+='*'+word+'*,'
                    search_string=search_string[:-1]

                try:
                    result = cf.find(name=search_string)
                    if len(result)==0:
                        returnlist='No PV found.'
                    else:
                        for pv in result:
                            returnlist+='<a href="#" onclick="searchPV(\''+pv['name']+'\')">'+pv['name']+'</a>'
                            returnlist+='<br>'
                except:
                    returnlist='Channel Finder Server Not Available.'

                self.wfile.write(bytes(json.dumps({'pvs':returnlist}), "utf-8"))
                return

            if path[0] == 'getPVHTML':
                self.send_header('Content-type','application/json')
                self.send_header('Access-Control-Allow-Origin', '*')
                self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
                self.end_headers()
                pvinfo={}
                try:
                    myPV=epics.PV(path[1])
                    pvinfo['info']=myPV.info.replace('\n','<br>')
                    myPV.clear_auto_monitor()
                    myPV.disconnect()
                    archiver_stat = ''
                    archiver_dict=archiver.get_pv_status(pv=path[1])[0]
                    for i in archiver_dict:
                        archiver_stat = archiver_stat + i + ' = '+ archiver_dict[i]+'\n'
                    pvinfo['archiver']=archiver_stat.replace('\n','<br>')
                    if archiver_dict['status'] == 'Being archived':
                        pvinfo['link']='<a target="_blank" href="/plot/retrieval/ui/viewer/archViewer.html?pv='+path[1]+'">See archived data</a>'

                    cf_dict = cf.find(name=path[1])[0]
                    pvinfo['channel_finder']="<table>"
                    pvinfo['channel_finder']+="<tr><td>PV Name:</td><td>"+cf_dict['name']+"</td></tr>"
                    pvinfo['channel_finder']+="<tr><td>Owner:</td><td>"+cf_dict['owner']+"</td></tr>"
                    pvinfo['channel_finder']+="<tr><td><p></td></tr>"
                    pvinfo['channel_finder']+="<tr><td>Properties</td></tr>"
                    pvinfo['channel_finder']+="<tr><td>Name</td><td>Owner</td><td>Value</td></tr>"
                    for prop in cf_dict['properties']:
                        pvinfo['channel_finder']+="<tr><td>"+prop['name']+"</td><td>"+prop['owner']+"</td><td>"+prop['value']+"</td></tr>"
                    pvinfo['channel_finder']+="<tr><td><p></td></tr>"
                    pvinfo['channel_finder']+="<tr><td>Tags</td></tr>"
                    for tag in cf_dict['tags']:
                        pvinfo['channel_finder']+="<tr><td>"+tag+"</tr></td>"
                    pvinfo['channel_finder']+="</table>"

                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return
                except:
                    pvinfo['info']="The PV seems disconnected"
                    pvinfo['archiver']=''
                    pvinfo['channel_finder']=''
                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return

            if path[0] == 'getPV':
                self.send_header('Content-type','application/json')
                self.send_header('Access-Control-Allow-Origin', '*')
                self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
                self.end_headers()
                pvinfo={}
                try:
                    if path[1]=='ISrc-010:PBI-BCM-001:AI4-Compressed-Mean':
                        myPV=epics.PV('ISrc-010:PBI-BCM-001:AI4-Compressed', auto_monitor=True)
                        mean_value=myPV.value.mean()
                        tmp_info = myPV.info.split('\n')
                        tmp_info[0]='== ISrc-010:PBI-BCM-001:AI4-Compressed-Mean  (time_double) =='
                        tmp_info[1]='   value      = '+str(mean_value)
                        tmp_info[2]="   char_value = '"+str(mean_value)+"'"
                        tmp_info[3]='   count      = 1'
                        tmp_info[4]='   nelm       = 1'
                        new_info=''
                        for i in tmp_info:
                            new_info+=i
                            new_info+='\n'
                        new_info=new_info[:-1]
                        myPV.clear_auto_monitor()
                        myPV.disconnect()
                        pvinfo['info']=new_info
                        pvinfo['archiver']="pvName = ISrc-010:PBI-BCM-001:AI4-Compressed-Mean\nstatus = Not being archived\n"
                        pvinfo['live_view']="ISrc-010:PBI-BCM-001:AI4-Compressed"
                        self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                        return
                    if path[1]=='ISrc-010:PBI-BCM-001:AI4-Compressed-Max':
                        myPV=epics.PV('ISrc-010:PBI-BCM-001:AI4-Compressed', auto_monitor=True)
                        max_value=myPV.value.max()
                        tmp_info = myPV.info.split('\n')
                        tmp_info[0]='== ISrc-010:PBI-BCM-001:AI4-Compressed-Max  (time_double) =='
                        tmp_info[1]='   value      = '+str(max_value)
                        tmp_info[2]="   char_value = '"+str(max_value)+"'"
                        tmp_info[3]='   count      = 1'
                        tmp_info[4]='   nelm       = 1'
                        new_info=''
                        for i in tmp_info:
                            new_info+=i
                            new_info+='\n'
                        new_info=new_info[:-1]
                        myPV.clear_auto_monitor()
                        myPV.disconnect()
                        pvinfo['info']=new_info
                        pvinfo['archiver']="pvName = ISrc-010:PBI-BCM-001:AI4-Compressed-Max\nstatus = Not being archived\n"
                        pvinfo['live_view']="ISrc-010:PBI-BCM-001:AI4-Compressed"
                        self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                        return

                    myPV=epics.PV(path[1])
                    pvinfo['info']=myPV.info
                    myPV.clear_auto_monitor()
                    myPV.disconnect()
                    archiver_stat = ''
                    archiver_dict=archiver.get_pv_status(pv=path[1])[0]
                    for i in archiver_dict:
                        archiver_stat = archiver_stat + i + ' = '+ archiver_dict[i]+'\n'
                    pvinfo['archiver']=archiver_stat
                    pvinfo['live_view']='https://live.pos.esss.lu.se/live_view?PV='+path[1]
                    if archiver_dict['status'] == 'Being archived':
                        pvinfo['link']='/plot/retrieval/ui/viewer/archViewer.html?pv='+path[1]
                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return
                except:
                    pvinfo['info']="Something wrong here, please retry"
                    pvinfo['archiver']="Something wrong here, please retry"
                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return
            return

        self.wfile.write(bytes('The server is running', "utf-8"))
        return


def on_exit(signum, frame):
    print(f'Signal handler called with signal {signum}')
    raise SystemExit("Exiting")

signal.signal(signal.SIGTERM, on_exit)

try:
    posThread = posScreen(stop_signal)
    posThread.start()
    interlocksThread = interlocksScreen(stop_signal)
    interlocksThread.start()
    instrumentsThread = instrumentsScreen(stop_signal)
    instrumentsThread.start()
    ts2_cavThread = ts2_cavScreen(stop_signal)
    ts2_cavThread.start()
    ts2_screensThread = ts2_screensScreen(stop_signal)
    ts2_screensThread.start()
    ts2_tpcircuitsThread = ts2_tpcircuitsScreen(stop_signal)
    ts2_tpcircuitsThread.start()
    
    server = HTTPServer(('', PORT_NUMBER), myHandler)
    print ('Started httpserver on port ' , PORT_NUMBER)
    server.serve_forever()

except (KeyboardInterrupt, SystemExit):
    print ('\n^C received, shutting down the web server')
    server.socket.close()
    stop_signal.set()
